package com.suke.czx.config;

import cn.hutool.core.util.ReUtil;
import com.suke.czx.common.annotation.XssIgnore;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.mvc.method.RequestMappingInfo;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerMapping;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

/**
 * @author czx
 * @title: AuthIgnoreConfig
 * @projectName x-springboot
 * @description: TODO
 * @date 2019/12/2415:56
 */
@Slf4j
@Configuration
public class XssIgnoreConfig implements InitializingBean {

    @Autowired
    private WebApplicationContext applicationContext;

    private static final Pattern PATTERN = Pattern.compile("\\{(.*?)\\}");
    private static final String ASTERISK = "*";

    @Getter
    @Setter
    private List<String> ignoreUrls = new ArrayList<>();

    @Override
    public void afterPropertiesSet(){
        RequestMappingHandlerMapping mapping = applicationContext.getBean("requestMappingHandlerMapping",RequestMappingHandlerMapping.class);
        Map<RequestMappingInfo, HandlerMethod> map = mapping.getHandlerMethods();
        map.keySet().forEach(mappingInfo -> {
            HandlerMethod handlerMethod = map.get(mappingInfo);
            XssIgnore method = AnnotationUtils.findAnnotation(handlerMethod.getMethod(), XssIgnore.class);
            if(method != null){
                mappingInfo.getPathPatternsCondition().getPatternValues().stream().forEach(url ->{
                    ignoreUrls.add(ReUtil.replaceAll(url, PATTERN, ASTERISK));
                });
            }
        });
    }

    public boolean isContains(String url){
        final String u = ReUtil.replaceAll(url, PATTERN, ASTERISK);
        return ignoreUrls.contains(u);
    }
}
